/*
 *  test/pem_test.dart
 *
 *  David Janes
 *  2019-03-15
 */

import "package:x509csr/x509csr.dart";
import "package:test/test.dart";
import "package:pointycastle/export.dart";

import "dart:typed_data";
import "dart:convert";

void main() {
  group("PEM", () {
    test("PEM Cycle", () {
      String pem = """\
-----BEGIN CERTIFICATE REQUEST-----\r
MIICpzCCAY8CAQAwYjEbMBkGA1UEAwwSd3d3LmRhdmlkamFuZXMuY29tMRIwEAYD\r
VQQKDAlDb25zZW5zYXMxEDAOBgNVBAcMB1Rvcm9udG8xEDAOBgNVBAgMB09udGFy\r
aW8xCzAJBgNVBAYTAkNBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA\r
o1lf0VDufSHwUfaiDYua2r+o/Eo7msA3/MMnJhtRnAGQKKPxVxcv0T+L+gAb48fw\r
b24oriiELaA3zVTK69FMiCK1AdSkR/KbRQUGtEShyAtaq0FAFY/1to2VJ+CG19mp\r
a31m+g6BHC1NGyV5bti1bgSvNFGC7+0YBJFtu/0VFDWN+QzPKEX7IEMkgzerr9u+\r
WzB7FvtIXdbPFrvZ+rHR+Z363uHgG4nTpQnChFtADDeibHtKqIAjLB7/oIT8XL6t\r
MTpIaKAYg9mMvRmorQTuEw6S57LqTWA2QX+9V2SaTD7n0TTb2EiBY1QE+ogZ7T+s\r
Rzkeqj5SMqlb1nyevJcv+wIDAQABoAAwDQYJKoZIhvcNAQEEBQADggEBAPsvl7ye\r
fNZbqTJSPqoeOUesP+0ZiPoEVGOBSNjbNNHnPkyaZFe9f0E2YE3qsueSDhPuBK2o\r
Gb2M2YMYoGhIOjGtvlz8hKD/HiwjgKhKe2yiNwxAW4TCCaXTiRvg4d76nfnRsfrZ\r
uxbP1l1I+xZ7MFu+26+rN4MkQyD7RSjPDPmNNRQV/bttkQQY7e+CUTSvBG612G55\r
JRtNLRyBDvpmfWup2deG4CeVjbb1jxVAQataC8ihRLQGBUWb8kek1AG1IohM0evK\r
VM03oC2EKK4obm/wx+MbAPqLP9EvF1fxoyiQAZxRGyYnw/w3wJo7Svyov9qaiw2i\r
9lHwIX3uUNFfWaM=\r
-----END CERTIFICATE REQUEST-----\r\n""";

      Uint8List decoded1 = decodeBytesFromPEM(pem);
      String encoded1 = base64.encode(decoded1);
      Uint8List decoded2 = decodeBytesFromPEM(encoded1);
      String encoded2 = base64.encode(decoded2);
      Uint8List decoded3 = decodeBytesFromPEM(encoded2);
      String encoded3 = base64.encode(decoded3);

      expect(encoded3, equals(encoded1));
    });

    test("Decode Public Key", () {
      String pem = """\
-----BEGIN PUBLIC KEY-----\r
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo1lf0VDufSHwUfaiDYua\r
2r+o/Eo7msA3/MMnJhtRnAGQKKPxVxcv0T+L+gAb48fwb24oriiELaA3zVTK69FM\r
iCK1AdSkR/KbRQUGtEShyAtaq0FAFY/1to2VJ+CG19mpa31m+g6BHC1NGyV5bti1\r
bgSvNFGC7+0YBJFtu/0VFDWN+QzPKEX7IEMkgzerr9u+WzB7FvtIXdbPFrvZ+rHR\r
+Z363uHgG4nTpQnChFtADDeibHtKqIAjLB7/oIT8XL6tMTpIaKAYg9mMvRmorQTu\r
Ew6S57LqTWA2QX+9V2SaTD7n0TTb2EiBY1QE+ogZ7T+sRzkeqj5SMqlb1nyevJcv\r
+wIDAQAB\r
-----END PUBLIC KEY-----\r\n""";
      RSAPublicKey publicKey = decodeRSAPublicKeyFromPEM(pem);
      expect(publicKey, isNotNull);
    });

    test("Decode Private Key", () {
      String pem = """\
-----BEGIN PRIVATE KEY-----\r
MIIEvAIBADANBgkqhkiG9w0BAQEFAASCBKYwggSiAgEAAoIBAQCjWV/RUO59IfBR\r
9qINi5rav6j8SjuawDf8wycmG1GcAZAoo/FXFy/RP4v6ABvjx/BvbiiuKIQtoDfN\r
VMrr0UyIIrUB1KRH8ptFBQa0RKHIC1qrQUAVj/W2jZUn4IbX2alrfWb6DoEcLU0b\r
JXlu2LVuBK80UYLv7RgEkW27/RUUNY35DM8oRfsgQySDN6uv275bMHsW+0hd1s8W\r
u9n6sdH5nfre4eAbidOlCcKEW0AMN6Jse0qogCMsHv+ghPxcvq0xOkhooBiD2Yy9\r
GaitBO4TDpLnsupNYDZBf71XZJpMPufRNNvYSIFjVAT6iBntP6xHOR6qPlIyqVvW\r
fJ68ly/7AgMBAAECggEAXwr9iVjBquU4YzhRifgUnfquZDz8+UmmSddyy+VRUqhG\r
LJ9XKQlAtUt4MnolwB7xcqRx3wOMBNAYJ8ySVStukDdBglMrUwGB/mKyR+k3CKIy\r
mzpYAzPED/fJNW6ahrUKspRAenY+ARsKpiTjxu8ogV8QYiybNuaI9v93Dc5vmVWx\r
0WMqyo3uaXLQQpuMGJABlwuxR0cyEHwMbEfUDaxvHdmqCORQl1OXeWqmffhuMjBq\r
suPagdvWACZzREhsdrjYKt2PDhJvIwfYidGn/BU72dJEbzUlPkNIIblxpjnlsb8W\r
CM9l9PEpKfQLIZUoHTzA5rN0WPWeZS+0GPYSeCM2AQKBgQDNT2VLVhZb6gDeLJ7T\r
6JzKhwujMA9++Pb/AJjLfWL/K+S1+6j5kRZfH31iZvbD0wCTmhe7JACbNBmynS0K\r
f+AMCIry1k7TRavC0okubSG+BxNTbg9EHtLROzwofNEQoFdAyJkdHwHxl44DJ3l+\r
tZDjCrz5MVy8hbkhbmboUZ3LuwKBgQDLrdZERdCbDcEz2mXvrX39zEReLeuJNwUp\r
KMWhBatwGXlNrbaRVdDVugioIiBGVixPb/p4oFti7+hiBX5y0mNE8aY3xBhOCouX\r
y50dG6BSSsdniBN+TrhRd8rgZYW8PXnEW+ZvO70bvROAMFihszuKdIAL7mGn+KA/\r
gSAHQrxIwQKBgGH7iz8qBZ+2DNX7e41sjS+GxcIK1MpnLRHECifLXmyjWRKU2S7J\r
1GBvWdqVgx0v3S2UM9EnPHbVpZH9uxowrw9vIEkGiDYCIRfofHnAsYLMQ6fkdbqv\r
7zLVzz8PWoNR0nJjiddBDh9rkeXld/FF27DPViKjMTvzocfXtZsYIHOzAoGASFHT\r
MK6xYTzDCwxTAYVSpRkwdVhMt43nsTOe6IvA0a7Z2XEC0BvuaNUDWd+0IgVa+mHC\r
R654hGq20ocs3MxDHynGYSqpjpxD8IQFp1410MJohyRT95cv7w1f5clmcO5LHCOd\r
cIV/oifCktOXxOKFE6ZGCPZ9ikzHaYUJwEo73IECgYBOpq7yHxbUg0ITBQGqwrCJ\r
CHFUFefpmq85M8B31OVZaZEqBgyMWB2iWb8kavY8KH4RiyUouEyTD3a0bty0/pvO\r
78+wUie57BQtiCmSRF8okl9NeouGpewy6djU36aQnyoPNfbgi8DBioGPW0NHAuR2\r
VIpR9QlvA3fRVEdb+RBM3A==\r
-----END PRIVATE KEY-----\r\n""";
      RSAPrivateKey privateKey = decodeRSAPrivateKeyFromPEM(pem);
      expect(privateKey, isNotNull);
    });
  });
}
